class Metadata:
    name = "cat9kthousandeyesctl"
    author = "Robert Csapo"
    email = "rcsapo@cisco.com"
    description = "Cisco Catalyst 9000 with Thousand Eyes agents"
    repo_url = "https://github.com/robertcsapo/cat9kthousandeyesctl"
    version = "0.0.1"
    license = "CISCO SAMPLE CODE LICENSE"
    copyright = "Copyright (c) 2021 Cisco and/or its affiliates."
